import { maybeRemoveFirst } from './array/maybeRemoveFirst';
import { forEachProp } from './object/forEachProp';
export class SimpleCache {
    constructor(opts) {
        this.ttlIds = {};
        if (opts) {
            this.ttl = opts.ttl;
            this.maxSize = opts.maxSize;
            this.store = opts.initialStore ? Object.assign({}, opts.initialStore) : {};
            this._keys = opts.initialKeys
                ? [...opts.initialKeys]
                : Object.keys(this.store);
        }
        else {
            this.store = {};
            this._keys = [];
        }
    }
    get size() {
        return this._keys.length;
    }
    get(key) {
        return this.store[key];
    }
    has(key) {
        return this.store.hasOwnProperty(key);
    }
    set(key, value, opts) {
        const { store, ttlIds, _keys } = this;
        if (store.hasOwnProperty(key)) {
            if (_keys[_keys.length - 1] !== key) {
                maybeRemoveFirst(_keys, key);
                _keys.push(key);
            }
        }
        else {
            maybeRemoveFirst(_keys, key);
            _keys.push(key);
            if (this.maxSize && _keys.length > this.maxSize) {
                this.delete(_keys.shift());
            }
        }
        store[key] = value;
        const tid = ttlIds[key];
        if (tid != null) {
            delete ttlIds[key];
            clearTimeout(tid);
        }
        let ttl = opts && opts.ttl;
        if (ttl == null)
            ttl = this.ttl;
        if (ttl != null) {
            ttlIds[key] = setTimeout(() => this.delete(key), ttl);
        }
    }
    delete(key) {
        if (this.store.hasOwnProperty(key)) {
            delete this.store[key];
            maybeRemoveFirst(this._keys, key);
            const tid = this.ttlIds[key];
            if (tid != null) {
                delete this.ttlIds[key];
                clearTimeout(tid);
            }
            return true;
        }
        return false;
    }
    clear() {
        this.store = {};
        this._keys = [];
        forEachProp(this.ttlIds, (id) => clearTimeout(id));
        this.ttlIds = {};
    }
    forEach(fn) {
        const { store, _keys } = this;
        for (let i = 0, len = _keys.length; i < len; i += 1) {
            const key = _keys[i];
            fn(store[key], key);
        }
    }
    getKeys() {
        return this._keys.slice();
    }
    getValues() {
        return this._keys.map((key) => this.store[key]);
    }
}
//# sourceMappingURL=SimpleCache.js.map